import 'package:perspective_api/src/models/attribute_score.dart';
import 'package:perspective_api/src/models/language.dart';

import 'requested_attribute.dart';

class AnalyzeCommentResponse {
  /// The requested languages.
  final Set<Language> languages;

  /// The auto-detected languages.
  final Set<Language> detectedLanguages;

  /// The attribute scores for this comment.
  final List<AttributeScore> attributeScores;

  const AnalyzeCommentResponse({
    required this.languages,
    required this.detectedLanguages,
    required this.attributeScores,
  });

  static AnalyzeCommentResponse parseAnalyzeCommentResponse(Map<String, Object?> res) {
    final {'languages': languages as List, 'detectedLanguages': detectedLanguages as List, 'attributeScores': rawAttributeScores as Map} = res;

    final attributeScores = <AttributeScore>[];

    for (final ra in rawAttributeScores.keys) {
      final attribute = RequestedAttribute.fromScreamingSnakeCase(ra);

      final attributeScore = AttributeScore.parseAttributeScore(rawAttributeScores[ra] as Map<String, Object?>, attribute);

      attributeScores.add(attributeScore);
    }

    return AnalyzeCommentResponse(
      languages: languages.map((l) => Language.fromCode(l)).toSet(),
      detectedLanguages: detectedLanguages
          .map((dl) {
            try {
              return Language.fromCode(dl);
            } on ArgumentError {
              return null;
            }
          })
          .whereType<Language>()
          .toSet(),
      attributeScores: attributeScores,
    );
  }
}
