import 'package:perspective_api/src/models/requested_attribute.dart';

class AttributeScore {
  /// The requested attribute.
  final RequestedAttribute attribute;

  /// The summary score for this attribute.
  final double summaryScore;

  /// The span scores for this attribute.
  final List<SpanScore> spanScores;

  const AttributeScore({
    required this.attribute,
    required this.summaryScore,
    required this.spanScores,
  });

  static AttributeScore parseAttributeScore(Map<String, Object?> res, RequestedAttribute attribute) {
    final {
      'summaryScore': sum as Map,
      'spanScores': spanScores as List,
    } = res;

    final summaryScore = sum['value'] as double;

    return AttributeScore(
      attribute: attribute,
      summaryScore: summaryScore,
      spanScores: spanScores.map((ss) => SpanScore.parseSpanScore(ss)).toList(),
    );
  }
}

class SpanScore {
  /// The beginning of the span.
  final int begin;

  /// The end of the span.
  final int end;

  /// The score for this span.
  final double score;

  // /// The type of the span.
  // final String type;

  const SpanScore({
    required this.begin,
    required this.end,
    required this.score,
    // required this.type,
  });

  static SpanScore parseSpanScore(Map<String, Object?> res) {
    final {
      'begin': begin as int,
      'end': end as int,
      'score': sc as Map,
      // 'type': type as String,
    } = res;

    final score = sc['value'] as double;

    return SpanScore(
      begin: begin,
      end: end,
      score: score,
      // type: type,
    );
  }
}
