import 'package:meta/meta.dart';

enum RequestedAttribute {
  toxicity,
  severeToxicity,
  identityAttack,
  insult,
  profanity,
  threat,

  // Experimental attributes.
  @experimental
  toxicityExperimental,
  @experimental
  severeToxicityExperimental,
  @experimental
  identityAttackExperimental,
  @experimental
  insultExperimental,
  @experimental
  profanityExperimental,
  @experimental
  threatExperimental,
  @experimental
  sexuallyExplicit,
  @experimental
  flirtation,

  // N-Y-T attributes.
  attackOnAuthor,
  attackOnCommenter,
  incoherent,
  inflammatory,
  likelyToReject,
  obscene,
  spam,
  unsubstantial;

  String toScreamingSnakeCase() {
    final split = name.split('');
    StringBuffer res = StringBuffer();

    for (final char in split) {
      if (char != char.toLowerCase() && res.isNotEmpty) {
        res.write('_');
        res.write(char);
      } else {
        res.write(char);
      }
    }

    return res.toString().toUpperCase();
  }

  static RequestedAttribute fromScreamingSnakeCase(String snakeCase) {
    final split = snakeCase.split('_');
    StringBuffer res = StringBuffer();

    for (final (i, word) in split.map((e) => e.toLowerCase()).indexed) {
      if (i == 0) {
        res.write(word);
      } else {
        res.write(word[0].toUpperCase());
        res.write(word.substring(1));
      }
    }

    return RequestedAttribute.values.firstWhere(
      (e) => e.name == res.toString(),
      orElse: () => throw ArgumentError.value(
        snakeCase,
        'snakeCase',
        'Invalid attribute name.',
      ),
    );
  }

  static const Set<RequestedAttribute> restrictedLanguages = {
    attackOnAuthor,
    attackOnCommenter,
    incoherent,
    inflammatory,
    likelyToReject,
    obscene,
    spam,
    unsubstantial,
  };
}
