import 'dart:convert';

import 'package:http/retry.dart';
import 'package:http/http.dart';
import 'package:perspective_api/src/http/request.dart';

import 'models/analyze_comment_response.dart';
import 'models/language.dart';
import 'models/requested_attribute.dart';

DateTime? _lastRequestTime;

final class PerspectiveApi {
  final String _apiKey;

  final _client = RetryClient(Client(), when: (res) {
    return _lastRequestTime == null ||
        _lastRequestTime!.isAfter(
          DateTime.now().subtract(
            const Duration(seconds: 1),
          ),
        );
  });

  PerspectiveApi({
    /// The API key to use for requests.
    required String apiKey,
  }) : _apiKey = apiKey;

  /// Analyze a comment.
  Future<AnalyzeCommentResponse> analyzeComment(
    String text, {
    required Set<RequestedAttribute> requestedAttributes,
    required Set<Language> languages,
  }) async {
    final body = {
      'comment': {'text': text},
      'requestedAttributes': {
        for (final ra in requestedAttributes)
          ra.toScreamingSnakeCase(): {},
      },
      'languages': languages.map((l) => l.code).toList(),
    };

    final request = BasicRequest(
      'comments:analyze',
      queryParameters: {
        'key': _apiKey,
      },
      body: json.encode(body),
      method: 'POST',
    );

    final res = await _client.send(request.prepare());

    _lastRequestTime = DateTime.now();

    final resBody = (await res.stream.bytesToString());

    final jsonBody = json.decode(resBody);

    return AnalyzeCommentResponse.parseAnalyzeCommentResponse(jsonBody);
  }
}
